/* wptCommonDlg.cpp
 *	Copyright (C) 2004, 2005, 2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <ctype.h>

#include "resource.h"

#include "wptTypes.h"
#include "wptW32API.h"
#include "wptVersion.h"
#include "wptGPG.h"
#include "wptCommonCtl.h"
#include "wptContext.h"


/* check if the given buf contains a valid URL. */
static int
check_URL (const char *buf)
{
    size_t i;
    const char *allowed = "-./#_:";

    if (strncmp (buf, "http://", 7))
	return -1;
    if (!strchr (buf, ':'))
	return -1;
    if (!strstr (buf, "//"))
	return -1;
    if (strstr (buf, " "))
	return -1;
    buf += 7;
    for (i=0; i < strlen (buf); i++) {
	if (isalpha (buf[i]) || isdigit (buf[i]))
	    continue;
	if (!strchr (allowed, buf[i]))
	    return -1;
    }

    return 0;
}


static BOOL CALLBACK
http_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static struct URL_ctx_s * ctx;
    int n;

    switch (msg) {
    case WM_INITDIALOG:
	ctx = (struct URL_ctx_s*)lparam;
	if (!ctx)
	    BUG (0);
	if (ctx->desc != NULL)
	    SetWindowText (dlg, ctx->desc);
	if (ctx->title != NULL)
	    SetDlgItemText (dlg, IDC_HTTP_TITLE, ctx->title);
	SetDlgItemText (dlg, IDCANCEL, _("&Cancel"));
	SetForegroundWindow (dlg);
	break;

    case WM_COMMAND:
	switch (LOWORD (wparam)) {
	case IDOK:
	    n = GetDlgItemText (dlg, IDC_HTTP_URL, ctx->url, DIM (ctx->url)-1);
	    if (ctx->check && (!n || check_URL (ctx->url))) {
		msg_box (dlg, _("Please enter a valid URL."), "HTTP", MB_ERR);
		return FALSE;
	    }
	    ctx->cancel = 0;
	    EndDialog (dlg, TRUE);
	    break;

	case IDCANCEL:
	    ctx->cancel = 1;
	    EndDialog (dlg, FALSE);
	    break;
	}
    }

    return FALSE;
}


/* Common dialog to request a URL from the user. 
   Return value is the URL context. */
void*
get_http_file_dlg (HWND root)
{
    struct URL_ctx_s *ctx;

    ctx = new URL_ctx_s;
    if (!ctx)
	BUG (0);
    memset (ctx, 0, sizeof *ctx);
    ctx->check = 1;
    ctx->desc = _("HTTP Key Import");
    ctx->title = _("Enter URL to retrieve the public key");
    DialogBoxParam (glob_hinst, (LPCTSTR)IDD_WINPT_HTTP, root,
		    http_dlg_proc, (LPARAM)ctx);
    return ctx;
}

/* Wrapper to request a preferred keyserver from the user.
   Return value is the URL context. */
void*
get_keyserver_URL_dlg (HWND root)
{
    struct URL_ctx_s *ctx;

    ctx = new URL_ctx_s;
    if (!ctx)
	BUG (0);
    memset (ctx, 0, sizeof *ctx);
    ctx->check = 1;
    ctx->desc = _("Key Edit");
    ctx->title = _("Enter preferred keyserver URL"); 
    DialogBoxParam (glob_hinst, (LPCTSTR)IDD_WINPT_HTTP, root,
		    http_dlg_proc, (LPARAM)ctx);
    return ctx;
}


/* Generic text input dialog. */
char*
get_input_dialog (HWND root, const char *title, const char *desc)
{
    struct URL_ctx_s ctx;

    memset (&ctx, 0, sizeof (ctx));
    ctx.title = title;
    ctx.desc = desc;
    DialogBoxParam (glob_hinst, (LPCTSTR)IDD_WINPT_HTTP, root,
		    http_dlg_proc, (LPARAM)&ctx);
    if (ctx.cancel || strlen (ctx.url) == 0)
	return NULL;
    return m_strdup (ctx.url);
}

#if 0
struct infodlg_s {
    const char *title;
    const char *button_1;
    const char *button_2;
    const char *text;
    HICON	ico;
    int		retval;
};

/* Dialog box procedure for the info dialog. */
static BOOL CALLBACK
infodlg_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static struct infodlg_s *ctx;

    switch (msg) {
    case WM_INITDIALOG:
	ctx = (struct infodlg_s *)lparam;
	if (!ctx)
	    EndDialog (dlg, FALSE);
	if (ctx->title)
	    SetWindowText (dlg, ctx->title);
	if (ctx->text)
	    SetDlgItemText (dlg, IDC_INFODLG_TEXT, ctx->text);
	if (ctx->button_1)
	    SetDlgItemText (dlg, IDOK, ctx->button_1);
	if (ctx->button_2)
	    SetDlgItemText (dlg, IDCANCEL, ctx->button_2);
	if (ctx->ico != NULL) {
	    SendMessage (GetDlgItem (dlg, IDC_INFODLG_ICO), STM_SETICON,
			 (WPARAM)(HICON)ctx->ico, 0);
	}
	SetForegroundWindow (dlg);
	return TRUE;

    case WM_COMMAND:
	switch (LOWORD (wparam)) {
	case IDOK:
	    ctx->retval = IDOK;
	    EndDialog (dlg, TRUE);
	    break;

	case IDCANCEL:
	    ctx->retval = IDCANCEL;
	    EndDialog (dlg, FALSE);
	    break;
	}
	break;
    }
    return FALSE;
}


int
InfoBox (HWND hwnd, LPCTSTR text, LPCTSTR title, UINT type)
{
    struct infodlg_s inf;

    memset (&inf, 0, sizeof (inf));
    inf.text = text;
    inf.title = title;
    if (type & MB_YESNO) {
	inf.button_1 = _("Yes");
	inf.button_2 = _("No");
    }
    else {
	inf.button_1 = _("OK");
	inf.button_2 = _("Cancel");
    }
    if (type & MB_ICONERROR)
	inf.ico = LoadIcon (NULL, IDI_ERROR);
    else if (type & MB_ICONWARNING)
	inf.ico = LoadIcon (NULL,  IDI_EXCLAMATION);
    else if (type & MB_ICONINFORMATION)
	inf.ico = LoadIcon (NULL, IDI_INFORMATION);
    DialogBoxParam (glob_hinst, (LPCTSTR)IDD_WINPT_INFODLG, hwnd, 
		    infodlg_dlg_proc, (LPARAM)&inf);
    return inf.retval;
}


int
InfoBoxEx (HWND hwnd, LPCTSTR text, LPCTSTR title, UINT type, ...)
{      
    va_list arg_ptr;
    char buf[2048];
    int id;

    va_start (arg_ptr, type);
    _vsnprintf (buf, sizeof (buf)-1, text, arg_ptr);    
    id = InfoBox (hwnd, buf, title, type);
    va_end (arg_ptr);
    return id;
}
#endif

